import { TabTracker } from "./TabTracker.mjs";
import { RuleManager } from "./NetRequestRuleManager.mjs";
import { MultiRegexMatcher } from "./MultiRegexMatcher.mjs";
import { SponsorBlockIntegration } from "./SponsorBlockIntegration.mjs";
import { BackgroundUtils } from "./BackgroundUtils.mjs";
import { MessageTypes } from "../player/enums/MessageTypes.mjs";
import { URLUtils } from "../player/utils/URLUtils.mjs";
import { Utils } from "../player/utils/Utils.mjs";
import { PlayerModes } from "../player/enums/PlayerModes.mjs";
import { StreamSaverBackend } from "./StreamSaverBackend.mjs";
import { EnvUtils } from "../player/utils/EnvUtils.mjs";
import { StringUtils } from "../player/utils/StringUtils.mjs";

let Options = {};
const OptionsCache = {};
const AutoEnableList = [];
const ExtensionVersion = chrome.runtime.getManifest().version;
const Logging = false;
const Tabs = new TabTracker();
const ruleManager = new RuleManager();
let CustomSourcePatternsMatcher = new MultiRegexMatcher();
const sponsorBlockBackend = new SponsorBlockIntegration();
sponsorBlockBackend.setup();
try {
    sponsorBlockBackend.setup(self);
} catch (e) {
    console.error(e);
}
BackgroundUtils.openWelcomePageOnInstall();

BackgroundUtils.queryTabs().then((ctabs) => {
    ctabs.forEach((tabobj) => {
        const tab = Tabs.getTabOrCreate(tabobj.id);
        try {
            BackgroundUtils.updateTabIcon(tab, true);
        } catch (e) {
            console.error(e);
        }
    });
});

console.log("TabTracker---", Tabs);

async function onClicked(tabobj, open) {
    const tab = Tabs.getTabOrCreate(tabobj.id);
    console.log("onClicked:tab", tab);
    // check permissions
    const hasPerms = await BackgroundUtils.checkPermissions();
    if (!hasPerms) {
        chrome.tabs.create({
            url: chrome.runtime.getURL("perms.html"),
        });
        return;
    }

    if (tabobj.url) {
        tab.url = tabobj.url;
    }

    const emptyTabURLS = [
        "about:blank",
        "about:home",
        "about:newtab",
        "about:privatebrowsing",
        "chrome://newtab/",
    ];

    if (tab.url && !emptyTabURLS.includes(tab.url)) {
        if (!BackgroundUtils.isUrlPlayerUrl(tab.url)) {
            tab.isOn = open;

            BackgroundUtils.updateTabIcon(tab);

            if (tab.isOn) {
                openPlayersWithSources(tab);
            } else {
                let hasPlayer = false;
                for (const frame of tab.getFrames()) {
                    if (frame.isPlayer) {
                        hasPlayer = true;
                        break;
                    }
                }
                console.log("hasPlayer", hasPlayer);

                if (hasPlayer) {
                    tab.reset();
                    chrome.tabs.reload(tab.tabId);
                }
            }
        } else {
            tab.isOn = open;
            BackgroundUtils.updateTabIcon(tab);
        }
    } else {
        // chrome.tabs.update(
        //     tab.tabId,
        //     {
        //         url: BackgroundUtils.getPlayerUrl(),
        //     },
        //     () => {}
        // );
    }
}

// chrome.action.onClicked.addListener(onClicked);
chrome.tabs.onUpdated.addListener((tabid, changeInfo, tabobj) => {
    const tab = Tabs.getTabOrCreate(tabid);

    if (changeInfo.url && tabobj.active) {
        console.log("onUpdated: tab", tab);
        console.log("onUpdated: changeInfo", changeInfo);
        console.log("onUpdated: tabobj", tabobj);
        console.log("onUpdated: tabs", Tabs);
        const url = new URL(changeInfo.url);
        const oldURL = tab.url ? new URL(tab.url) : null;
        console.log('reset:oldURL.hostname', oldURL)
        console.log('reset:url.hostname', url)
        if (oldURL && oldURL.hostname !== url.hostname) {
            tab.reset();
        }
        console.log('url.href.includes(oldURL.href)', url.href.includes(oldURL.href));
        
        if (oldURL && !oldURL.href.includes(url.href)) {
            tab.url = changeInfo.url;
            chrome.tabs.reload(tabid);
        }
        return
        chrome.tabs.sendMessage(
            tabid,
            {
                type: MessageTypes.REMOVE_PLAYERS,
            },
            {
                frameId: 0,
            },
            () => {
                BackgroundUtils.checkMessageError("remove_players");
            }
        );
        console.log("after REMOVE_PLAYERS");

        const match = AutoEnableList.find((item) => {
            try {
                if (!item.regex) {
                    return (
                        changeInfo.url.substring(0, item.match.length) ===
                        item.match
                    );
                } else if (item.exclude_domain) {
                    return item.domain === url.hostname;
                } else {
                    return item.match.test(changeInfo.url);
                }
            } catch (e) {}
            return false;
        });

        const shouldAutoEnable = match && !match.negative;

        if (BackgroundUtils.isUrlPlayerUrl(tab.url)) {
            tab.isOn = true;
            tab.regexMatched = true;
        } else if (shouldAutoEnable && !tab.regexMatched) {
            tab.regexMatched = true;
            tab.isOn = true;
            openPlayersWithSources(tab);
        } else if (!shouldAutoEnable && tab.regexMatched) {
            tab.isOn = false;
            tab.regexMatched = false;
        }
    }

    BackgroundUtils.updateTabIcon(tab, true);
});
chrome.runtime.onMessage.addListener((msg, sender, sendResponse) => {
    if (msg.type === MessageTypes.POPUP_LOADED) {
        chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
            onClicked(tabs[0], true);
        });
        sendResponse("Popup loaded");
        return;
    } else if (msg.type === MessageTypes.POPUP_UNLOADED) {
        chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
            onClicked(tabs[0], false);
        });
        sendResponse("Popup unloaded");
        return;
    } else if (msg.type === MessageTypes.PING) {
        sendResponse(MessageTypes.PONG);
        return;
    }
    const tab = Tabs.getTabOrCreate(sender.tab.id);
    const frame = tab.getFrameOrCreate(sender.frameId);
    if (msg.type === MessageTypes.PLAYER_LOADED) {
        frame.isPlayer = true;

        if (tab.downloadInfo) {
            chrome.tabs.sendMessage(
                frame.tab.tabId,
                {
                    type: MessageTypes.HANDLE_DOWNLOAD,
                    url: tab.downloadInfo.url,
                    filename: tab.downloadInfo.filename,
                },
                {
                    frameId: frame.frameId,
                },
                (response) => {
                    BackgroundUtils.checkMessageError("download");
                    tab.downloadInfo.resolve(response);
                    tab.downloadInfo = null;

                    // Close tab
                    chrome.tabs.remove(frame.tab.tabId);
                }
            );
            sendResponse(null);
            return;
        }

        if (!frame.parent && msg.parentFrameId !== undefined) {
            const parentFrame = tab.getFrameOrCreate(msg.parentFrameId);
            frame.setParentFrame(parentFrame);
        }

        if (frame.playerOpening) {
            frame.playerOpening = false;
        } else if (frame.parent) {
            frame.parent.playerOpening = false;
        }
        tab.playerCount++;
        const isMainPlayer = tab.playerCount === 1;

        getPageFrame(frame).then((pageFrame) => {
            if (pageFrame) {
                frame.pageFrame = pageFrame;
            } else {
                frame.pageFrame = tab.getFrameOrCreate(0);
            }

            const response = {
                mediaInfo: getMediaInfoFromTab(sender?.tab),
                analyzerData: tab.analyzerData,
                isMainPlayer,
            };

            sendResponse(response);
        });
        return true;
    } else if (msg.type === MessageTypes.FRAME_ADDED) {
        const playerCount = frame.resetSelfAndChildren();
        frame.url = msg.url;
        tab.playerCount -= playerCount;
        tab.playerCount = Math.max(0, tab.playerCount);
        checkURLMatch(frame);
        frame.loadedCallbacks.forEach((callback) => {
            try {
                callback("loaded");
            } catch (e) {
                console.error(e);
            }
        });
        frame.loadedCallbacks.clear();
    } else if (msg.type === MessageTypes.FRAME_REMOVED) {
        const toRemove =
            msg.frameId !== undefined ? tab.getFrame(msg.frameId) : frame;
        const playerCount = toRemove.resetSelfAndChildren();
        tab.playerCount -= playerCount;
        tab.playerCount = Math.max(0, tab.playerCount);

        if (toRemove.parent) {
            toRemove.parent.removeChildFrame(toRemove);
        }

        tab.removeFrame(toRemove.frameId);
    } else if (msg.type === MessageTypes.WAIT_UNTIL_MAIN_LOADED) {
        frame.loadedCallbacks.add(sendResponse);

        // Try to ping tab
        chrome.tabs.sendMessage(
            tab.tabId,
            {
                type: MessageTypes.PING_TAB,
            },
            (response) => {
                BackgroundUtils.checkMessageError("ping_tab");
                if (response === MessageTypes.PONG_TAB) {
                    if (frame.loadedCallbacks.has(sendResponse)) {
                        frame.loadedCallbacks.delete(sendResponse);
                        sendResponse("loaded");
                    }
                }
            }
        );
        return true;
    } else if (msg.type === MessageTypes.SEND_TO_CONTENT) {
        chrome.tabs.sendMessage(
            tab.tabId,
            {
                type: MessageTypes.MESSAGE_FROM_CONTENT,
                data: msg.data,
                destination: msg.destination,
            },
            {
                frameId: frame.frameId,
            },
            (response) => {
                BackgroundUtils.checkMessageError("message_from_content");
            }
        );
    } else if (msg.type === MessageTypes.SEND_TO_PLAYER) {
        const pframe = tab.getFrame(msg.frameId);
        if (!pframe || !pframe.isPlayer) {
            sendResponse(null);
            return;
        }

        chrome.tabs.sendMessage(
            pframe.tab.tabId,
            {
                type: MessageTypes.MESSAGE_FROM_PAGE,
                data: msg.data,
            },
            {
                frameId: pframe.frameId,
            },
            (response) => {
                BackgroundUtils.checkMessageError("message_from_page");

                sendResponse(response);
            }
        );
        return true;
    } else if (msg.type === MessageTypes.REQUEST_SOURCES) {
        sendSources(frame);
    } else if (msg.type === MessageTypes.SET_HEADERS) {
        if (msg.commands.length) {
            ruleManager
                .addHeaderRule(msg.url, sender.tab.id, msg.commands)
                .then((rule) => {
                    sendResponse();
                });
            return true;
        }
    } else if (msg.type === MessageTypes.YT_LOADED) {
        frame.url = msg.url;
        checkYTURL(frame);
    }
});
function checkYTURL(frame) {
    const url = frame.url;
    // Check if url is youtube
    if (BackgroundUtils.isUrlPlayerUrl(url)) {
        return;
    }

    if (URLUtils.is_url_yt(url) && URLUtils.is_url_yt_watch(url)) {
        onSourceRecieved(
            {
                url: url,
                requestId: -1,
            },
            frame,
            PlayerModes.ACCELERATED_YT
        );
    }
}
function collectSources(frame, remove = false) {
    const subtitles = [];
    const sources = [];

    let currentFrame = frame;
    let removed = false;
    let depth = 0;

    while (currentFrame) {
        const currentSubtitles = currentFrame.getSubtitles();
        for (const sub of currentSubtitles) {
            subtitles.push({
                ...sub,
                depth,
                frameId: currentFrame.frameId,
            });
        }

        const currentSources = currentFrame.getSources();
        for (const source of currentSources) {
            sources.push({
                ...source,
                depth,
                frameId: currentFrame.frameId,
            });
        }

        if (!removed && remove) {
            if (currentSources.length !== 0) {
                removed = true;
            }
            currentSources.length = 0;
            currentSubtitles.length = 0;
        }

        if (sources.length !== 0) break;

        currentFrame = currentFrame.parent;
        depth++;
    }

    // Sort by time, oldest first
    sources.sort((a, b) => {
        return a.time - b.time;
    });

    return { subtitles, sources };
}
function sendSources(frame) {
    const { subtitles, sources } = collectSources(frame, true);

    const continuationOptions = frame.tab.continuationOptions;
    frame.tab.continuationOptions = null;

    chrome.tabs.sendMessage(
        frame.tab.tabId,
        {
            type: MessageTypes.SOURCES,
            subtitles: subtitles,
            sources: sources,
            autoSetSource: true,
            continuationOptions: continuationOptions,
        },
        {
            frameId: frame.frameId,
        },
        () => {
            BackgroundUtils.checkMessageError("sources");
        }
    );
}
function getMediaInfoFromTab(tab) {
    if (!tab || BackgroundUtils.isUrlPlayerUrl(tab.url)) return;
    // Get name of website through tab url
    const url = new URL(tab.url);
    const hostname = url.hostname;
    let name = hostname.split(".");
    name = name[name.length - 2];

    if (!name) return;

    let title = tab.title || "";

    // First, remove any special characters
    title = title.replace(/[^a-zA-Z0-9 ]/g, "");

    // Remove any words too similar to the website name
    const words = title.split(" ").filter((word) => word.length > 0);
    for (let i = 0; i < words.length; i++) {
        const word = words[i];
        if (word.length <= 3) {
            continue;
        }

        if (
            StringUtils.levenshteinDistance(
                word.toLowerCase(),
                name.toLowerCase()
            ) < Math.ceil(name.length * 0.4)
        ) {
            words.splice(i, 1);
            i--;

            // Check if previous word is "the" or "a" or etc...
            if (i > 0) {
                const prewordList = [
                    "the",
                    "a",
                    "an",
                    "of",
                    "and",
                    "or",
                    "in",
                    "on",
                    "at",
                    "to",
                    "for",
                    "with",
                    "by",
                    "from",
                ];
                if (prewordList.includes(words[i].toLowerCase())) {
                    words.splice(i, 1);
                    i--;
                }
            }
        }
    }

    title = words.join(" ");

    let season = null;
    let episode = null;
    // Remove season #, episode #, s#, e#
    title = title.replace(/\bseason\s*([0-9]+)/gi, (match, p1) => {
        season = parseInt(p1);
        return "";
    });

    title = title.replace(/\bepisode\s*([0-9]+)/gi, (match, p1) => {
        episode = parseInt(p1);
        return "";
    });

    if (season === null) {
        title = title.replace(/\bs([0-9]+)/gi, (match, p1) => {
            season = parseInt(p1);
            return "";
        });
    }

    if (episode === null) {
        title = title.replace(/\be([0-9]+)/gi, (match, p1) => {
            episode = parseInt(p1);
            return "";
        });
    }

    // Remove year
    title = title.replace(/\b[0-9]{4}\b/g, "");

    // Remove words like TV, Movie, HD, etc...
    // List generated by AI
    const wordsToExclude = [
        "tv",
        "movie",
        "hd",
        "full",
        "free",
        "online",
        "stream",
        "streaming",
        "watch",
        "now",
        "watching",
        "series",
        "episode",
        "season",
        "anime",
        "show",
        "shows",
        "episodes",
        "seasons",
        "part",
        "parts",
        "sub",
        "dub",
        "subdub",
        "subbed",
        "dubbed",
        "english",
        "subtitles",
        "subtitle",
    ];
    title = title.replace(
        new RegExp("\\b(" + wordsToExclude.join("|") + ")\\b", "gi"),
        ""
    );

    return {
        name: title.replace(/\s\s+/g, " ").trim(),
        season: season,
        episode: episode,
    };
}
async function getPageFrame(frame) {
    let currentFrame = frame;
    while (currentFrame && currentFrame.parent) {
        if (!currentFrame.linkPromise) {
            currentFrame.linkPromise = linkToParentFrame(currentFrame);
        }

        await currentFrame.linkPromise;

        const isFull = await checkIsFull(currentFrame);
        if (!isFull) {
            return currentFrame.parent;
        }
        currentFrame = currentFrame.parent;
    }

    return null;
}
async function checkIsFull(frame) {
    return new Promise((resolve, reject) => {
        chrome.tabs.sendMessage(
            frame.tab.tabId,
            {
                type: MessageTypes.IS_FULL,
                frameId: frame.frameId,
            },
            {
                frameId: frame.parent.frameId,
            },
            (response) => {
                BackgroundUtils.checkMessageError("is_full");
                resolve(response);
            }
        );
    });
}
async function linkToParentFrame(frame) {
    if (!frame.parent) return;

    // Generate random string
    const key = crypto.randomUUID();

    return await new Promise((resolve, reject) => {
        chrome.tabs.sendMessage(
            frame.tab.tabId,
            {
                type: MessageTypes.FRAME_LINK_RECEIVER,
                frameId: frame.frameId,
                key,
            },
            {
                frameId: frame.parent.frameId,
            },
            (response) => {
                BackgroundUtils.checkMessageError("link_frame_reciever");
                chrome.tabs.sendMessage(
                    frame.tab.tabId,
                    {
                        type: MessageTypes.FRAME_LINK_SENDER,
                        key,
                    },
                    {
                        frameId: frame.frameId,
                    },
                    (response) => {
                        BackgroundUtils.checkMessageError("link_frame_sender");
                        resolve();
                    }
                );
            }
        );
    });
}
function checkURLMatch(frame) {
    const url = frame.url;
    const ext = CustomSourcePatternsMatcher.match(url);

    if (ext) {
        const mode = URLUtils.getModeFromExtension(ext);

        if (!mode) return;
        onSourceRecieved(
            {
                url: url,
                requestId: -1,
            },
            frame,
            mode
        );
    }
}
async function openPlayersWithSources(tab) {
    let framesWithSources = [];
    for (const frame of tab.getFrames()) {
        if (!frame.isPlayer && frame.getSources().length > 0) {
            framesWithSources.push(frame);
        }
    }
    if (framesWithSources.length > 0) {
        framesWithSources = await Promise.all(
            framesWithSources.map(async (frame) => {
                return { frame, videoSize: await getVideoSize(frame) };
            })
        );

        framesWithSources.sort((a, b) => {
            return b.videoSize - a.videoSize;
        });

        for (let i = 0; i < framesWithSources.length; i++) {
            openPlayer(framesWithSources[i].frame);
        }
    } else {
        chrome.runtime.sendMessage({
            type: MessageTypes.NO_VIDEOS,
        });
    }
}
async function getVideoSize(frame) {
    return new Promise((resolve, reject) => {
        chrome.tabs.sendMessage(
            frame.tab.tabId,
            {
                type: MessageTypes.GET_VIDEO_SIZE,
            },
            {
                frameId: frame.frameId,
            },
            (size) => {
                BackgroundUtils.checkMessageError("get_video_size");
                resolve(size);
            }
        );
    });
}
async function openPlayer(frame) {
    if (frame.playerOpening || frame.hasPlayer()) {
        return;
    }

    frame.playerOpening = true;

    return new Promise((resolve, reject) => {
        chrome.tabs.sendMessage(
            frame.tab.tabId,
            {
                type: MessageTypes.OPEN_PLAYER,
                url: BackgroundUtils.getPlayerUrl(),
                noRedirect: frame.frameId === 0,
                frameId: frame.frameId,
                parentFrameId: frame.parent ? frame.parent.frameId : -1,
            },
            {
                frameId: frame.frameId,
            },
            (response) => {
                BackgroundUtils.checkMessageError("player");

                if (response === "no_video") {
                    frame.playerOpening = false;
                }

                resolve(response);
            }
        );
    });
}
function handleSubtitles(url, frame, headers) {
    const subtitles = frame.getSubtitles();
    if (
        subtitles.find((a) => {
            return a.source === url;
        })
    )
        return;

    const u = new URL(url).pathname.split("/").pop();

    subtitles.push({
        source: url,
        headers: headers,
        label: u.split(".")[0],
        time: Date.now(),
    });
}
function getSourceFromURL(frame, url) {
    return frame.getSources().find((a) => {
        return a.url === url;
    });
}
function addSource(frame, url, mode, headers) {
    frame.getSources().push({
        url,
        mode,
        headers,
        time: Date.now(),
    });
}
async function scrapeCaptionsTags(frame) {
    const tabId = frame.tab.tabId;
    const frameId = frame.frameId;
    if (tabId < 0) return null;

    return new Promise((resolve, reject) => {
        chrome.tabs.sendMessage(
            tabId,
            {
                type: MessageTypes.SCRAPE_CAPTIONS,
            },
            {
                frameId: frameId,
            },
            (sub) => {
                BackgroundUtils.checkMessageError("scrape_captions");
                resolve(sub);
            }
        );
    });
}
async function sendSourcesToMainFramePlayers(frame) {
    // query all tabs
    const tabs = await BackgroundUtils.queryTabs();

    // for each tab
    for (let i = 0; i < tabs.length; i++) {
        const tab = Tabs.getTab(tabs[i].id);
        if (!tab || !tab.isOn) continue;
        const mainPlayerFrame = tab.getMainPlayer();
        if (mainPlayerFrame) {
            // send the source to the tab
            chrome.tabs.sendMessage(
                tab.tabId,
                {
                    type: MessageTypes.SOURCES,
                    subtitles: frame.getSubtitles(),
                    sources: frame.getSources(),
                    autoSetSource: false,
                },
                () => {
                    BackgroundUtils.checkMessageError("sources");
                }
            );
        }
    }
}
async function onSourceRecieved(details, frame, mode) {
    if (
        (URLUtils.is_url_yt(frame.url) || URLUtils.is_url_yt(frame.tab.url)) &&
        mode !== PlayerModes.ACCELERATED_YT
    ) {
        return;
    }

    const url = details.url;
    const customHeaders =
        details.customHeaders || frame.requestHeaders.get(details.requestId);

    if (getSourceFromURL(frame, url)) return;

    // Check if service worker
    if (frame.tab.tabId < 0 && details.initiator) {
        // get current frame
        const currentFrame = await new Promise(async (resolve, reject) => {
            try {
                const ctabs = await BackgroundUtils.queryTabs();
                ctabs.every((ctab) => {
                    const tab = Tabs.getTab(ctab.id);
                    if (!tab) return true;

                    for (const frame of tab.getFrames()) {
                        const furl = frame.url;
                        if (
                            furl &&
                            furl.length >= details.initiator.length &&
                            furl.substring(0, details.initiator.length) ===
                                details.initiator
                        ) {
                            resolve(frame);
                            return false;
                        }
                    }
                    return true;
                });
            } catch (e) {
                resolve(null);
                return;
            }
        });

        if (currentFrame) {
            frame = currentFrame;
        }
    }

    addSource(frame, url, mode, customHeaders);

    const subs = await scrapeCaptionsTags(frame);
    if (subs) {
        subs.forEach((s) => {
            if (
                frame.subtitles.every((ss, i) => {
                    if (s.source === ss.source) {
                        frame.subtitles[i] = s;
                        return false;
                    }
                    return true;
                })
            )
                frame.subtitles.push(s);
        });
    }

    if (frame.tab.isOn) {
        clearTimeout(frame.openTimeout);
        frame.openTimeout = setTimeout(() => {
            if (frame.tab.isOn) {
                openPlayer(frame);
            }
        }, Options.replaceDelay);
    }

    sendSourcesToMainFramePlayers(frame);

    return;
}
const webRequestPerms = ["requestHeaders"];
const webRequestPerms2 = [];
if (EnvUtils.isChrome()) {
    webRequestPerms.push("extraHeaders");
    webRequestPerms2.push("extraHeaders");
}
chrome.webRequest.onBeforeRequest.addListener(
    (details) => {
        const tab = Tabs.getTabOrCreate(details.tabId);
        const frame = tab.getFrameOrCreate(details.frameId);
        if (!frame.parent && details.parentFrameId !== -1) {
            const parentFrame = tab.getFrameOrCreate(details.parentFrameId);
            frame.setParentFrame(parentFrame);
        }
    },
    {
        urls: ["<all_urls>"],
    }
);
chrome.webRequest.onBeforeSendHeaders.addListener(
    (details) => {
        const tab = Tabs.getTabOrCreate(details.tabId);
        const frame = tab.getFrameOrCreate(details.frameId);
        frame.requestHeaders.set(details.requestId, details.requestHeaders);
    },
    {
        urls: ["<all_urls>"],
    },
    webRequestPerms
);
const initiatorBlacklist = [
    "https://www.facebook.com",
    "https://www.instagram.com",
    "https://vimeo.com",
];

chrome.webRequest.onHeadersReceived.addListener(
    (details) => {
        const url = details.url;
        let ext = URLUtils.get_url_extension(url);
        const tab = Tabs.getTabOrCreate(details.tabId);
        const frame = tab.getFrameOrCreate(details.frameId);
        if (frame.hasPlayer()) return;

        if (
            (details.statusCode >= 400 && details.statusCode < 600) ||
            details.statusCode === 204
        ) {
            return; // Client or server error. Ignore it
        }

        if (
            details.initiator &&
            initiatorBlacklist.some((a) => {
                return details.initiator.startsWith(a);
            })
        ) {
            if (
                url.startsWith("https://vod-adaptive") &&
                url.includes("playlist.json")
            ) {
                ext = "vmpatch";
                // chrome.tabs.sendMessage(details.tabId, {
                //   type: 'vmurl',
                //   url: url,
                //   headers: frame.requestHeaders.get(details.requestId),
                // }, {
                //   frameId: details.frameId,
                // }, () => {
                //   BackgroundUtils.checkMessageError('vmurl');
                // });
                // return;
            } else if (ext === "json") {
            } else {
                return;
            }
        }

        const output = CustomSourcePatternsMatcher.match(url);
        if (output) {
            ext = output;
        }

        if (BackgroundUtils.isSubtitles(ext)) {
            return handleSubtitles(
                url,
                frame,
                frame.requestHeaders.get(details.requestId)
            );
        } else if (ext === "json") {
            // Vimeo. Check if filename is master.json
            const filename = URLUtils.get_file_name(url);
            if (filename === "master.json" && url.includes("/video/")) {
                ext = "mpd";
                details.url = URLUtils.strip_queryhash(url).replace(
                    "master.json",
                    "master.mpd"
                );
            }
        }

        let mode = URLUtils.getModeFromExtension(ext);
        if (!mode) {
            if (details.type === "media") {
                mode = PlayerModes.ACCELERATED_MP4;
            } else {
                // Check url query parameters
                const urlParams = URLUtils.get_url_params(url).values();
                for (const value of urlParams) {
                    if (!URLUtils.is_url(value)) continue;
                    let ext2 = URLUtils.get_url_extension(value);

                    const output = CustomSourcePatternsMatcher.match(value);
                    if (output) {
                        ext2 = output;
                    }

                    mode = URLUtils.getModeFromExtension(ext2);
                    if (mode) {
                        break;
                    }
                }
                if (!mode) {
                    return;
                }
            }
        }

        onSourceRecieved(details, frame, mode);
    },
    {
        urls: ["<all_urls>"],
    },
    webRequestPerms2
);

chrome.webRequest.onHeadersReceived.addListener(deleteHeaderCache, {
    urls: ["<all_urls>"],
});

chrome.webRequest.onErrorOccurred.addListener(deleteHeaderCache, {
    urls: ["<all_urls>"],
});

function deleteHeaderCache(details) {
    const tab = Tabs.getTabOrCreate(details.tabId);
    const frame = tab.getFrameOrCreate(details.frameId);
    frame.requestHeaders.delete(details.requestId);
}

async function loadOptions(newOptions) {
    newOptions = newOptions || (await Utils.getOptionsFromStorage());
    Options = newOptions;

    AutoEnableList.length = 0;
    Options.autoEnableURLs.forEach((urlStr) => {
        if (urlStr.length === 0) {
            return;
        }

        const obj = {
            exclude_domain: false,
            negative: false,
            regex: false,
            match: null,
        };

        while (urlStr.length > 0) {
            if (urlStr[0] === "!") {
                obj.negative = true;
                urlStr = urlStr.substring(1);
            } else if (urlStr[0] === "~") {
                obj.regex = true;
                urlStr = urlStr.substring(1);
            } else if (urlStr[0] === "-") {
                obj.exclude_domain = true;
                urlStr = urlStr.substring(1);
            } else {
                break;
            }
        }

        if (obj.exclude_domain) {
            try {
                // Check if starts with http or https, add it if not
                if (!urlStr.startsWith("http")) {
                    urlStr = "http://" + urlStr;
                }

                obj.domain = new URL(urlStr).hostname;
            } catch (e) {}
        } else if (obj.regex) {
            try {
                obj.match = new RegExp(urlStr);
            } catch (e) {}
        } else {
            obj.match = urlStr;
        }

        if (obj.match) {
            AutoEnableList.push(obj);
        }
    });

    // Reverse auto enable list
    AutoEnableList.reverse();

    if (Options.playMP4URLs) {
        setupRedirectRule(1, ["mp4"]);
    } else {
        removeRule(1);
    }

    if (Options.playStreamURLs) {
        setupRedirectRule(2, ["m3u8", "mpd"]);
    } else {
        removeRule(2);
    }

    loadCustomPatterns();
}
async function setupRedirectRule(ruleID, filetypes) {
    const excludedRequestDomains = [
        new URL(BackgroundUtils.getPlayerUrl()).hostname,
    ];

    AutoEnableList.forEach((item) => {
        if (
            item.exclude_domain &&
            item.domain &&
            !excludedRequestDomains.includes(item.domain)
        ) {
            excludedRequestDomains.push(item.domain);
        }
    });

    const rule = {
        id: ruleID,
        action: {
            type: "redirect",
            redirect: {
                regexSubstitution: BackgroundUtils.getPlayerUrl() + "#\\0",
            },
        },
        condition: {
            // exclude self
            excludedRequestDomains,
            // only match m3u8 or mpds
            regexFilter: "^.+\\.(" + filetypes.join("|") + ")([\\?|#].*)?$",
            resourceTypes: ["main_frame"],
        },
    };
    return chrome.declarativeNetRequest.updateDynamicRules({
        removeRuleIds: [rule.id],
        addRules: [rule],
    });
}
async function removeRule(ruleID) {
    return chrome.declarativeNetRequest.updateDynamicRules({
        removeRuleIds: [ruleID],
    });
}
async function loadCustomPatterns() {
    const customSourcePatterns = Options.customSourcePatterns;
    if (OptionsCache.customSourcePatterns !== customSourcePatterns) {
        OptionsCache.customSourcePatterns = customSourcePatterns;

        const matcher = new MultiRegexMatcher();
        await loadCustomPatternsFile(
            matcher,
            Options.customSourcePatterns,
            true
        );

        if (OptionsCache.customSourcePatterns !== customSourcePatterns) return;

        matcher.compile();
        CustomSourcePatternsMatcher = matcher;
    }
}
async function loadCustomPatternsFile(matcher, fileStr, isPrimary = false) {
    const lines = fileStr.split("\n");
    for (let i = 0; i < lines.length; i++) {
        const line = lines[i].trim();

        if (line.length === 0) continue;

        if (line.startsWith("#") || line.startsWith("//")) continue; // comment

        if (line.startsWith("@")) {
            // command, do nothing for now. Future use
            // line = line.substring(1);
            // const command = line.substring(0, line.indexOf(' '));
            // const body = line.substring(line.indexOf(' ') + 1);
            continue;
        }

        const args = line.split(" ");
        const extStr = args[0];
        const regexStr = args.slice(1).join(" ");

        // parse regex and flags
        const regex = regexStr.substring(1, regexStr.lastIndexOf("/"));
        const flags = regexStr.substring(regexStr.lastIndexOf("/") + 1);

        try {
            matcher.addRegex(regex, flags, extStr);
        } catch (e) {
            console.warn(e);
        }
    }
}
async function pingContentScript() {
    return new Promise(async (resolve, reject) => {
        const tabs = await BackgroundUtils.queryTabs();
        const message = {
            type: MessageTypes.PING_TAB,
        };

        // try one tab at a time. If it fails, try the next one
        for (const tab of tabs) {
            const result = await new Promise((resolve, reject) => {
                try {
                    chrome.tabs.sendMessage(
                        tab.id,
                        message,
                        (returnMessage) => {
                            if (
                                chrome.runtime.lastError ||
                                returnMessage !== MessageTypes.PONG_TAB
                            ) {
                                resolve(false);
                            } else {
                                resolve(true);
                            }
                        }
                    );
                } catch (e) {
                    resolve(false);
                }
            });

            if (result) {
                resolve(true);
                return;
            }
        }
        resolve(false);
    });
}

loadOptions().catch(console.error);
setInterval(async () => {
    await chrome.runtime.getPlatformInfo();
    await pingContentScript();
}, 10e3);
const streamSaverBackend = new StreamSaverBackend();
try {
    streamSaverBackend.setup(self);
} catch (e) {}
if (EnvUtils.isChrome()) {
    chrome.action.setBadgeBackgroundColor({
        color: [56, 114, 223, 255],
    });
}
